"""Test cases for generating files."""
from pathlib import Path

import pytest
from sphinx.testing.util import SphinxTestApp


@pytest.mark.sphinx("revealjs", testroot="default")
def test_not_generate_genindex(app: SphinxTestApp, status, warning):  # noqa
    app.build()
    assert not (app.outdir / "genindex.html").exists()


@pytest.mark.sphinx("revealjs", testroot="default")
def test_not_generate_search(app: SphinxTestApp, status, warning):  # noqa
    app.build()
    assert not (app.outdir / "search.html").exists()


@pytest.mark.sphinx("revealjs", testroot="default")
def test_contains_scss(app: SphinxTestApp, status, warning):  # noqa
    app.build()
    assert Path(app.outdir / "_static/revealjs4/css").exists()
    assert Path(
        app.outdir / "_static/revealjs4/css/theme/template/settings.scss"
    ).exists()



"""Definition for sphinx custom builder."""
import copy
import logging
from typing import Any, Dict, List, Tuple

from sphinx import version_info as sphinx_versoin
from sphinx.application import Sphinx
from sphinx.builders.dirhtml import DirectoryHTMLBuilder
from sphinx.builders.html import StandaloneHTMLBuilder
from sphinx.config import Config
from sphinx.environment import BuildEnvironment
from sphinx.locale import __

from sphinx_revealjs.directives import raw_json
from sphinx_revealjs.writers import RevealjsSlideTranslator

from .contexts import RevealjsPlugin, RevealjsProjectContext
from .utils import static_resource_uri

logger = logging.getLogger(__name__)


class RevealjsHTMLBuilder(StandaloneHTMLBuilder):
    """Sphinx builder class to generate Reveal.js presentation HTML.

    This manage theme path and configure default options.
    """

    name = "revealjs"
    default_translator_class = RevealjsSlideTranslator
    search = False

    def __init__(self, app, env: BuildEnvironment = None):  # noqa: D107
        # TODO: Remove it if this not need support Sphinx 5.1.0 and older
        if sphinx_versoin[0] <= 4 or sphinx_versoin[:2] == (5, 0):
            super().__init__(app)
        else:
            super().__init__(app, env)
        self.revealjs_slide = None

    def init(self):  # noqa
        # Create RevealjsProjectContext
        self.revealjs_context = RevealjsProjectContext(
            4,
            [  # noqa: W503
                static_resource_uri(src)
                for src in getattr(self.config, "revealjs_script_files", [])
            ],
            getattr(self.config, "revealjs_script_conf", None),
            [
                RevealjsPlugin(
                    static_resource_uri(plugin["src"]),
                    plugin.get("name", ""),
                    plugin.get("options", "{}").strip(),
                )
                for plugin in getattr(self.config, "revealjs_script_plugins", [])
            ],
        )
        # Hand over builder configs to html builder.
        setattr(self.config, "html_static_path", self.config.revealjs_static_path)
        super().init()
        self.use_index = self.get_builder_config("use_index", "revealjs")

    def init_css_files(self) -> None:  # noqa
        self.add_css_file(self.revealjs_context.engine.css_path)
        for filename in self.get_builder_config("css_files", "revealjs"):
            self.add_css_file(filename)

    def init_js_files(self) -> None:  # noqa
        for filename, attrs in self.app.registry.js_files:
            self.add_js_file(filename, **attrs)

        for filename, attrs in self.get_builder_config("js_files", "revealjs"):
            self.add_js_file(filename, **attrs)

    def get_theme_config(self) -> Tuple[str, Dict]:
        """Find and return configuration about theme (name and option params).

        Find theme and merge options.
        """
        theme_name = getattr(self.config, "revealjs_theme", "sphinx_revealjs")
        theme_options = getattr(self.config, "revealjs_theme_options", {})
        config = raw_json(theme_options.get("revealjs_config", ""))
        theme_options["revealjs_config"] = config
        return theme_name, theme_options

    def get_doc_context(self, docname, body, metatags):
        """Return customized context.

        if source has ``revealjs_slide`` property, add configures.
        """
        ctx = super().get_doc_context(docname, body, metatags)
        ctx["css_files"] = copy.copy(self.css_files)
        ctx["revealjs"] = self.revealjs_context
        return ctx

    def update_page_context(
        self, pagename: str, templatename: str, ctx: Dict, event_arg: Any
    ) -> None:  # noqa
        self.configure_theme(ctx)
        ctx["revealjs_page_confs"] = self.configure_page_script_conf()

    def configure_theme(self, ctx: Dict):
        """Find and add theme css from conf and directive."""
        # Use directive or conf
        if self.revealjs_slide and "theme" in self.revealjs_slide.attributes:
            theme = self.revealjs_slide.attributes["theme"]
        else:
            theme = self.config.revealjs_style_theme
        # Build path of stylesheet
        if theme.startswith("http://") or theme.startswith("https://"):
            pass
        elif theme.endswith(".css"):
            theme = f"_static/{theme}"
        else:
            theme = f"_static/{self.revealjs_context.engine.theme_dir}/{theme}.css"
        # index 0: "_static/revealjs4/dist/reveal.css"
        # index 1: theme css file path
        # index 2 or later: other css files
        ctx["css_files"].insert(1, theme)

    def configure_page_script_conf(self) -> List[str]:  # noqa
        if not self.revealjs_slide:
            return []
        configs = []
        if "conf" in self.revealjs_slide.attributes:
            configs.append(self.revealjs_slide.attributes["conf"])
        if self.revealjs_slide.content:
            configs.append(self.revealjs_slide.content)
        return configs


class DirectoryRevealjsHTMLBuilder(DirectoryHTMLBuilder, RevealjsHTMLBuilder):
    """Custom RevealjsHTMLBuilder to generate all HTML pages as ``index.html``.

    This does not have specific features, only inherit base builders.
    """

    name = "dirrevealjs"


def convert_reveal_js_files(app: Sphinx, config: Config) -> None:
    """Convert string styled html_js_files to tuple styled one.

    Original is :py:func:`sphinx.builders.html.convert_html_js_files`.
    """
    revealjs_js_files = []  # type: List[Tuple[str, Dict]]
    for entry in config.revealjs_js_files:
        if isinstance(entry, str):
            revealjs_js_files.append((entry, {}))
        else:
            try:
                filename, attrs = entry
                revealjs_js_files.append((filename, attrs))
            except Exception:
                logger.warning(__("invalid js_file: %r, ignored"), entry)
                continue
    config.revealjs_js_files = revealjs_js_files  # type: ignore



"""Custom docutils nodes for Reveal.js."""
from docutils import nodes


class FlagAttribute(object):
    """Flag options for docutils node."""

    pass


class SectionTagRenderer(object):
    """Mix-in class to build attributes combined string."""

    def attributes_str(self):
        """Build string of attributes for Reveal.js sections.

        Catch only keys starting 'data-'.
        Others are skipped.
        """
        pair = []
        for k, v in self.attributes.items():
            if not k.startswith("data-"):
                continue
            if isinstance(v, FlagAttribute):
                pair.append(k)
                continue
            pair.append(f'{k}="{v}"')
        return " ".join(pair)


class revealjs_section(
    SectionTagRenderer, nodes.Structural, nodes.Element
):  # noqa: D101,E501
    pass


class revealjs_break(
    SectionTagRenderer, nodes.Structural, nodes.Element
):  # noqa: D101,E501
    pass


class revealjs_slide(nodes.Structural, nodes.Element):  # noqa: D101
    pass


class revealjs_fragments(nodes.Structural, nodes.Element):  # noqa: D101
    pass



"""Test cases for ``revealjs-section`` directive."""
import pytest
from sphinx.testing.util import SphinxTestApp
from testutils import soup_html


@pytest.mark.sphinx("revealjs", testroot="default")
def test_render_section1_bg(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "with-revealjs-section.html")
    section_tag = soup.h1.parent
    assert "data-background-color" in section_tag.attrs
    assert section_tag["data-background-color"] == "#000001"


@pytest.mark.sphinx("revealjs", testroot="default")
def test_render_section2_bg(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "with-revealjs-section.html")
    section_tag = soup.h2.parent
    assert "data-background-color" in section_tag.attrs
    assert section_tag["data-background-color"] == "#000101"


@pytest.mark.sphinx("revealjs", testroot="default")
def test_no_render_parent_bg(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "with-revealjs-section.html")
    section_tag = soup.h2.parent.parent
    assert "data-background-color" not in section_tag.attrs



"""Custom directives for Reveal.js."""
import json

from docutils.nodes import Sequential
from docutils.parsers.rst import Directive, directives
from sphinx.util import logging

from sphinx_revealjs.nodes import (
    FlagAttribute,
    revealjs_break,
    revealjs_fragments,
    revealjs_section,
    revealjs_slide,
)

logger = logging.getLogger(__name__)


def raw_json(argument):
    """Type of direction attribute."""
    if argument is None:
        return directives.unchanged(argument)
    try:
        json.loads(argument)
    except json.decoder.JSONDecodeError:
        return ""
    return argument


REVEALJS_SECTION_ATTRIBUTES = {
    # Markup / Slide State
    "data-state": directives.unchanged,
    # Backgrounds / Color Backgrounds
    "data-background-color": directives.unchanged,
    # Backgrounds / Gradient Backgrounds
    "data-background-gradient": directives.unchanged,
    # Backgrounds / Image Backgrounds
    "data-background-image": directives.unchanged,
    "data-background-position": directives.unchanged,
    "data-background-repeat": directives.unchanged,
    # Backgrounds / Video Backgrounds
    "data-background-video": directives.unchanged,
    "data-background-video-loop": directives.unchanged,
    "data-background-video-muted": directives.unchanged,
    # Backgrounds / Image and Video Backgrounds
    "data-background-size": directives.unchanged,
    "data-background-opacity": directives.unchanged,
    # Backgrounds / Iframe Backgrounds
    "data-background-iframe": directives.unchanged,
    "data-background-interactive": lambda x: FlagAttribute(),
    # Slide Visibility
    "data-visibility": directives.unchanged,
    # Transitions
    "data-transition": directives.unchanged,
    "data-transition-speed": directives.unchanged,
    "data-background-transition": directives.unchanged,
    # Auto-Animate
    "data-auto-animate": lambda x: FlagAttribute(),
    "data-auto-animate-delay": directives.unchanged,
    "data-auto-animate-duration": directives.unchanged,
    "data-auto-animate-easing": directives.unchanged,
    "data-auto-animate-unmatched": directives.unchanged,
    "data-auto-animate-id": directives.unchanged,
    "data-auto-animate-restart": lambda x: FlagAttribute(),
    # Auto-Slide / Slide Timing
    "data-autoslide": directives.unchanged,
}


class RevealjsSection(Directive):  # noqa: D101
    option_spec = REVEALJS_SECTION_ATTRIBUTES

    def run(self):  # noqa: D102
        node = revealjs_section()
        node.attributes = self.options
        return [
            node,
        ]


class RevealjsBreak(Directive):  # noqa: D101
    option_spec = dict(
        # if it is set, next section does not display title
        notitle=lambda x: FlagAttribute(),
        **REVEALJS_SECTION_ATTRIBUTES
    )

    def run(self):  # noqa: D102
        node = revealjs_break()
        node.attributes = self.options
        return [
            node,
        ]


class RevealjsSlide(Directive):  # noqa: D101
    has_content = True

    option_spec = {
        "theme": directives.unchanged,
        "google_font": directives.unchanged,
        "conf": raw_json,
    }

    def run(self):  # noqa: D102
        node = revealjs_slide()
        node.attributes = self.options
        node.content = "\n".join(self.content or [])
        return [
            node,
        ]


class RevealjsFragments(Directive):  # noqa: D101
    has_content = True

    def run(self):  # noqa: D102
        node = revealjs_fragments()
        if self.content:
            self.state.nested_parse(self.content, self.content_offset, node)
        # TODO: Parameter ?
        for child in node.children:
            if not isinstance(child, Sequential):
                child["classes"].append("fragment")
                continue
            for c in child:
                c["classes"].append("fragment")
        return [
            node,
        ]



"""Test cases for comment block."""
import pytest
from sphinx.testing.util import SphinxTestApp
from testutils import soup_html


@pytest.mark.sphinx("revealjs", testroot="for-commented-notes")
def test_ignore_comment_at_default_config(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "index.html")
    assert len(soup.find_all("aside")) == 0


@pytest.mark.sphinx(
    "revealjs",
    testroot="for-commented-notes",
    confoverrides={"revealjs_notes_from_comments": True},
)
def test_comment_as_notes(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "index.html")
    assert len(soup.find_all("aside")) == 1



"""Build tests for configuration parameters."""
import pytest
from sphinx.testing.util import SphinxTestApp
from testutils import soup_html


@pytest.mark.sphinx(
    "revealjs",
    testroot="for-config",
    confoverrides={"revealjs_style_theme": "moon"},
)
def test_using_builtin_theme(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "index.html")
    links = [
        e
        for e in soup.find_all("link", rel="stylesheet")
        if "_static/revealjs4/dist/theme/moon.css" in e["href"]
    ]
    assert len(links) == 1


@pytest.mark.sphinx(
    "revealjs",
    testroot="for-config",
    confoverrides={"revealjs_style_theme": "custom.css"},
)
def test_using_custom_theme(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "index.html")
    links = [
        e
        for e in soup.find_all("link", rel="stylesheet")
        if "_static/custom.css" in e["href"]
    ]
    assert len(links) == 1



"""Test cases for ``revealjs-notes`` directive."""
import pytest
from sphinx.testing.util import SphinxTestApp
from testutils import soup_html


@pytest.mark.sphinx("revealjs", testroot="default")
def test_simple_notes(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "with-revealjs-notes/simple.html")
    assert len(soup.find_all("aside")) == 1
    notes = soup.find_all("aside", {"class": "notes"})
    assert len(notes) == 1
    assert len(notes[0]) == 1


@pytest.mark.sphinx("revealjs", testroot="default")
def test_html_included_notes(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "with-revealjs-notes/has-html.html")
    assert len(soup.find_all("aside")) == 1
    notes = soup.find_all("aside", {"class": "notes"})
    assert len(notes) == 1
    assert len(notes[0]) == 1


@pytest.mark.sphinx("html", testroot="default")
def test_no_out_by_html(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "with-revealjs-notes/simple.html")
    notes = soup.find_all("aside", {"class": "notes"})
    assert len(notes) == 0



"""Build tests for configuration parameters."""
import pytest
from sphinx.testing.util import SphinxTestApp
from testutils import soup_html


class TestForRevealjsCssFiles:  # noqa
    @pytest.mark.sphinx(
        "revealjs",
        testroot="for-config",
        confoverrides={"revealjs_css_files": ["https://example.com/example.css"]},
    )
    def test_url(self, app: SphinxTestApp, status, warning):  # noqa
        soup = soup_html(app, "index.html")
        links = [
            e
            for e in soup.find_all("link", rel="stylesheet")
            if "https://example.com/example.css" in e["href"]
        ]
        assert len(links) == 1

    @pytest.mark.sphinx(
        "revealjs",
        testroot="for-config",
        confoverrides={
            "revealjs_static_path": ["_static"],
            "revealjs_css_files": ["custom.css"],
        },
    )
    def test_localfile(self, app: SphinxTestApp, status, warning):  # noqa
        soup = soup_html(app, "index.html")
        links = [
            e
            for e in soup.find_all("link", rel="stylesheet")
            if "_static/custom.css" in e["href"]
        ]
        assert len(links) == 1
        assert (app.outdir / "_static/custom.css").exists()


class TestForRevealjsJsFiles:  # noqa
    @pytest.mark.sphinx(
        "revealjs",
        testroot="for-config",
        confoverrides={
            "revealjs_js_files": [
                "https://cdnjs.cloudflare.com/ajax/libs/dayjs/1.10.7/dayjs.min.js"
            ]
        },
    )
    def test_revealjs_js_files(self, app: SphinxTestApp, status, warning):  # noqa
        soup = soup_html(app, "index.html")
        elm = soup.find(
            "script",
            src="https://cdnjs.cloudflare.com/ajax/libs/dayjs/1.10.7/dayjs.min.js",
        )
        assert elm is not None


# TODO: Unsupport 2.x (keep until 3.x)
@pytest.mark.sphinx(
    "revealjs",
    testroot="for-config",
    confoverrides={
        "html_js_files": [
            "https://cdnjs.cloudflare.com/ajax/libs/dayjs/1.10.7/dayjs.min.js"
        ]
    },
)
def test_unuse_html_js_files(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "index.html")
    elm = soup.find(
        "script", src="https://cdnjs.cloudflare.com/ajax/libs/dayjs/1.10.7/dayjs.min.js"
    )
    assert elm is None
    assert soup.find("script", src="_static/jquery.js") is None
    assert soup.find("script", src="_static/underscore.js") is None



"""Test cases for ``revealjs-fragments`` directive."""
import pytest
from sphinx.testing.util import SphinxTestApp
from testutils import soup_html


@pytest.mark.sphinx("revealjs", testroot="default")
def test_list_fragments(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "with-revealjs-fragments.html")
    target = soup.find_all("h3")[0].parent
    assert len(target.find_all(attrs={"fragment"})) == 3


@pytest.mark.sphinx("revealjs", testroot="default")
def test_paragraph_fragments(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "with-revealjs-fragments.html")
    target = soup.find_all("h3")[1].parent
    assert len(target.find_all(attrs={"fragment"})) == 2



"""Test cases for ``revealjs-break`` directive."""
import pytest
from sphinx.testing.util import SphinxTestApp
from testutils import soup_html


@pytest.mark.sphinx("revealjs", testroot="default")
def test_on_revealjs_builder(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "with-revealjs-break.html")
    sec2 = soup.find_all("section")[1]
    # title, content1, content2 (splitted)
    assert len(sec2.find_all("section")) == 3



"""Root module for sphinx-revealjs."""

__version__ = "2.5.0"

import sys

from packaging.specifiers import SpecifierSet
from packaging.version import parse
from sphinx.application import Sphinx
from sphinx.config import Config
from sphinx.util import logging

from sphinx_revealjs.builders import (
    DirectoryRevealjsHTMLBuilder,
    RevealjsHTMLBuilder,
    convert_reveal_js_files,
)
from sphinx_revealjs.directives import (
    RevealjsBreak,
    RevealjsFragments,
    RevealjsSection,
    RevealjsSlide,
)
from sphinx_revealjs.nodes import (
    revealjs_break,
    revealjs_fragments,
    revealjs_section,
    revealjs_slide,
)
from sphinx_revealjs.themes import get_theme_path
from sphinx_revealjs.writers import (
    depart_revealjs_break,
    not_write,
    visit_revealjs_break,
)

logger = logging.getLogger(__name__)


def inherit_extension_nodes(app: Sphinx, config: Config):
    """Inherit behaviors of nodes from other sphinx extensions.

    .. note::

        I want to use add_node with override option,
        but sphinx app manage only names of nodes, not class types.
    """
    html_trans = app.registry.translation_handlers["html"]
    revealjs_trans = app.registry.translation_handlers["revealjs"]
    dirrevealjs_trans = app.registry.translation_handlers["dirrevealjs"]
    for n, b in html_trans.items():
        if n not in revealjs_trans:
            revealjs_trans[n] = b
    for n, b in revealjs_trans.items():
        if n not in dirrevealjs_trans:
            dirrevealjs_trans[n] = b


def notify_deprecated(app: Sphinx, config: Config):  # noqa: D103
    """Do not work. But it keep for next deprecated."""
    logger.info(
        "NOTICE: For next major version, path of Revealjs will change to other."
    )


def setup(app: Sphinx):
    """Set up function called by Sphinx."""
    # Message deprecated
    python_support = SpecifierSet(">=3.7.0")
    python_version = parse(sys.version.split(" ")[0])
    if python_version not in python_support:
        logger.warning("You are using not supported version Python.")

    app.connect("config-inited", inherit_extension_nodes)
    app.connect("config-inited", convert_reveal_js_files)
    app.connect("config-inited", notify_deprecated)
    app.add_builder(RevealjsHTMLBuilder)
    app.add_builder(DirectoryRevealjsHTMLBuilder)
    app.add_node(
        revealjs_section,
        html=(not_write, not_write),
        latex=(not_write, not_write),
        text=(not_write, not_write),
        man=(not_write, not_write),
        texinfo=(not_write, not_write),
        revealjs=(not_write, not_write),
        dirrevealjs=(not_write, not_write),
    )
    app.add_node(
        revealjs_break,
        html=(not_write, not_write),
        latex=(not_write, not_write),
        text=(not_write, not_write),
        man=(not_write, not_write),
        texinfo=(not_write, not_write),
        revealjs=(visit_revealjs_break, depart_revealjs_break),
        dirrevealjs=(visit_revealjs_break, depart_revealjs_break),
    )
    app.add_node(
        revealjs_slide,
        html=(not_write, not_write),
        latex=(not_write, not_write),
        text=(not_write, not_write),
        man=(not_write, not_write),
        texinfo=(not_write, not_write),
        revealjs=(not_write, not_write),
        dirrevealjs=(not_write, not_write),
    )
    app.add_node(
        revealjs_fragments,
        html=(not_write, not_write),
        latex=(not_write, not_write),
        text=(not_write, not_write),
        man=(not_write, not_write),
        texinfo=(not_write, not_write),
        revealjs=(not_write, not_write),
        dirrevealjs=(not_write, not_write),
    )
    app.add_directive("revealjs-section", RevealjsSection)
    app.add_directive("revealjs-break", RevealjsBreak)
    app.add_directive("revealjs-slide", RevealjsSlide)
    app.add_directive("revealjs-fragments", RevealjsFragments)
    app.add_config_value("revealjs_use_section_ids", False, True)
    app.add_config_value("revealjs_use_index", False, "env")
    app.add_config_value("revealjs_static_path", [], True)
    app.add_config_value("revealjs_style_theme", "black", True)
    app.add_config_value("revealjs_js_files", [], True)
    app.add_config_value("revealjs_css_files", [], True)
    app.add_config_value("revealjs_script_files", [], True)
    app.add_config_value("revealjs_script_conf", None, True)
    app.add_config_value("revealjs_script_plugins", [], True)
    app.add_html_theme("sphinx_revealjs", str(get_theme_path("sphinx_revealjs")))
    app.setup_extension("sphinx_revealjs._ext.highlightings")
    app.setup_extension("sphinx_revealjs._ext.notes")
    return {
        "version": __version__,
        "env_version": 1,
        "parallel_read_safe": False,
    }



"""Test support functions."""
from typing import AnyStr

from bs4 import BeautifulSoup
from sphinx.testing.util import SphinxTestApp


def soup_html(app: SphinxTestApp, path: str) -> BeautifulSoup:
    """Build application and parse content."""
    app.build()
    html: AnyStr = (app.outdir / path).read_text()
    return BeautifulSoup(html, "html.parser")



"""Test cases for ``revealjs_slide`` directive."""
import pytest
from sphinx.testing.util import SphinxTestApp
from testutils import soup_html


@pytest.mark.sphinx("revealjs", testroot="default")
def test_override_theme(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "with-revealjs-slide/has-theme.html")
    css_hrefs = [elm["href"] for elm in soup.find_all("link", rel="stylesheet")]
    assert "../_static/custom.css" in css_hrefs


@pytest.mark.sphinx("revealjs", testroot="default")
def test_config(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "with-revealjs-slide/has-conf.html")
    assert 'Object.assign(revealjsConfig, {"transition": "none"});' in str(
        soup.find_all("script")[-1]
    )


@pytest.mark.sphinx("revealjs", testroot="default")
def test_config_as_content(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "with-revealjs-slide/has-content.html")
    assert "Object.assign(revealjsConfig, {\n" in str(soup.find_all("script")[-1])
    assert '"transition": "none"\n' in str(soup.find_all("script")[-1])
    assert "});" in str(soup.find_all("script")[-1])



"""Test cases for ``revealjs-code-block`` directive."""
import pytest
from sphinx.testing.util import SphinxTestApp
from testutils import soup_html


@pytest.mark.sphinx("revealjs", testroot="default")
def test_inherit_code_block(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "with-revealjs-code-block.html")
    code_tag = soup.find_all("code")[0]
    assert "python" in code_tag.attrs["class"]


@pytest.mark.sphinx("revealjs", testroot="default")
def test_dataline(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "with-revealjs-code-block.html")
    code_tag = soup.find_all("code")[1]
    assert "php" in code_tag.attrs["class"]
    assert "data-line-numbers" in code_tag.attrs


@pytest.mark.sphinx("revealjs", testroot="default")
def test_dataline_start_from(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "with-revealjs-code-block.html")
    code_tag = soup.find_all("code")[2]
    assert "toml" in code_tag.attrs["class"]
    assert "data-line-numbers" in code_tag.attrs
    assert code_tag.attrs["data-ln-start-from"] == "5"



"""Test cases for builtin theme."""
import pytest
from sphinx.testing.util import SphinxTestApp
from testutils import soup_html


@pytest.mark.sphinx("revealjs", testroot="default")
def test_has_revealcss(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "index.html")
    reveal_css = [
        d
        for d in soup.find_all("link", rel="stylesheet")
        if d["href"].endswith("dist/reveal.css")
    ]
    assert len(reveal_css) == 1


@pytest.mark.sphinx("revealjs", testroot="default")
def test_script_conf(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "index.html")
    script = soup.find_all("script")[-1]
    assert "Reveal.initialize(revealjsConfig);" in str(script)


@pytest.mark.sphinx("revealjs", testroot="default")
def test_script_sources(app, status, warning):  # noqa
    soup = soup_html(app, "index.html")
    scripts = [s["src"] for s in soup.find_all("script") if "src" in s.attrs]
    assert "_static/revealjs4/dist/reveal.js" in scripts


@pytest.mark.sphinx("revealjs", testroot="default")
def test_stylesheet(app, status, warning):  # noqa
    soup = soup_html(app, "index.html")
    links = [link["href"] for link in soup.find_all("link", rel="stylesheet")]
    assert "_static/revealjs4/dist/theme/black.css" in links



"""Build tests for core behaviors of presentation(sections)."""
import pytest
from sphinx.testing.util import SphinxTestApp
from testutils import soup_html


@pytest.mark.sphinx("revealjs", testroot="default")
def test_render_title(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "index.html")
    assert soup.h1.text == "Test presentation"


@pytest.mark.sphinx("revealjs", testroot="default")
def test_count_horizontal_sections(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "index.html")
    main = soup.find("div", {"role": "main"})
    assert len(main.find_all("section", recursive=False)) == 3


@pytest.mark.sphinx("revealjs", testroot="default")
def test_count_vertical_sections(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "index.html")
    main = soup.find("div", {"role": "main"})
    section = main.find_all("section", reccursive=False)[1]
    assert len(section.find_all("section", recursive=False)) == 3



#!/usr/bin/env python
"""Fetch and sync reveal.js resources.

This script need to run these case.

* After editable install
* Before build package archibves
"""
import argparse
import json
import logging
import shutil
import sys
import tarfile
import tempfile
from pathlib import Path
from typing import Dict
from urllib.request import urlretrieve

ROOT_DIR = Path(__file__).parent.parent.absolute()

RULE = {
    "name": "reveal.js",
    "targets": {
        "package/css": "css",
        "package/dist": "dist",
        "package/plugin": "plugin",
        "package/LICENSE": "LICENSE",
    },
    "dest": "sphinx_revealjs/themes/sphinx_revealjs/static/revealjs4",
}


def find_package(src: Path, name: str) -> dict:
    """Pick package file URL from package-lock.json."""
    package_lock = json.loads(src.read_text())
    deps = [m for n, m in package_lock.get("dependencies", {}).items() if n == name]
    if len(deps) == 0:
        raise ValueError(f"Invalid name: ({name})")
    return deps[0]


def extract_archive(source: Path, dest: Path, targets: Dict[str, str]):  # noqa: D103
    extract_dir = Path(tempfile.mkdtemp())
    with tarfile.open(str(source)) as tr:
        for trf in tr.getmembers():
            is_target = False
            for name in targets.keys():
                if trf.name.startswith(f"{name}"):
                    is_target = True
                    continue
            if not is_target:
                continue
            tr.extract(trf, extract_dir)
    for s, d in targets.items():
        func = shutil.copytree if (extract_dir / s).is_dir() else shutil.copyfile
        func(extract_dir / s, dest / d)


def main(args: argparse.Namespace):  # noqa: D103
    dest = Path(RULE["dest"])
    if dest.exists() and not args.force:
        print("Dest directory is already exists")
        sys.exit(1)
    elif args.force:
        shutil.rmtree(dest)
    package_lock_json = ROOT_DIR / "package-lock.json"
    package = find_package(package_lock_json, RULE["name"])
    local_archive = ROOT_DIR / "var" / f"{RULE['name']}-{package['version']}.tgz"
    if not local_archive.exists():
        urlretrieve(package["resolved"], local_archive)
    extract_archive(local_archive, dest, RULE["targets"])


if __name__ == "__main__":
    if Path.cwd() != ROOT_DIR:
        logging.error("This script can run only project root.")
        sys.exit(1)
    (ROOT_DIR / "var").mkdir(exist_ok=True)
    parser = argparse.ArgumentParser()
    parser.add_argument("-f", "--force", action="store_true", default=False)
    args = parser.parse_args()
    main(args)



"""Util as functions for some modules."""


def static_resource_uri(src: str, prefix: str = None) -> str:
    """Build static path of resource."""
    local_prefix = "_static" if prefix is None else prefix
    if src.startswith("http://") or src.startswith("https://"):
        return src
    return f"{local_prefix}/{src}"


def deprecated_message(version: str, frm: str, to: str) -> str:  # noqa: D103
    return f"DEPRECATED: {frm} will drop by {version}, pleas use {to}"



"""Build tests for configuration parameters."""
import pytest
from sphinx.testing.util import SphinxTestApp
from testutils import soup_html


@pytest.mark.sphinx(
    "revealjs",
    testroot="for-config",
    confoverrides={
        "revealjs_use_section_ids": True,
    },
)
def test_inject_id_to_all_sections(app: SphinxTestApp, status, warning):  # noqa
    soup = soup_html(app, "index.html")
    for e in soup.find_all("section"):
        children = set([c.name for c in e.children])
        if children & {"h1", "h2", "h3"}:
            assert "id" in e.attrs


@pytest.mark.sphinx(
    "revealjs",
    testroot="for-config",
    confoverrides={
        "revealjs_use_section_ids": True,
    },
)
def test_inject_id_to_all_sections_with_label(
    app: SphinxTestApp, status, warning
):  # noqa
    soup = soup_html(app, "with-label.html")
    h2_section = soup.h2.parent
    assert "id" in h2_section.attrs
    assert h2_section["id"] == "override-label"



"""Contexts for passing between objects."""
import json
from typing import Dict, List, Union

from .utils import static_resource_uri


class RevealjsEngine:
    """Reveal.js core metadata."""

    def __init__(
        self, version: int, js_path: str, css_path: str, theme_dir: str
    ):  # noqa
        self.version = version
        self.js_path = js_path
        self.css_path = css_path
        self.theme_dir = theme_dir

    @classmethod
    def from_version(cls, version: int = 4):  # noqa
        return cls(
            version,
            "revealjs4/dist/reveal.js",
            "revealjs4/dist/reveal.css",
            "revealjs4/dist/theme",
        )


class RevealjsPlugin:
    """Plugin metadata."""

    def __init__(self, src: str, name: str = None, options: str = None):  # noqa
        self.src = src
        self.name = name
        self.options = options


class RevealjsProjectContext(object):
    """Context object for Reveal.js (project-wide)."""

    def __init__(
        self,
        engine_version: int,
        script_files: List[str] = None,
        script_conf: Union[str, Dict] = None,
        script_plugins: List[RevealjsPlugin] = None,
    ):  # noqa
        self.engine = RevealjsEngine.from_version(engine_version)
        if isinstance(script_conf, str):
            self.script_conf = script_conf
        else:
            self.script_conf = f"JSON.parse('{json.dumps(script_conf)}')"
        self.script_plugins = script_plugins or []
        self._script_files = script_files or []

    @property
    def script_files(self):  # noqa
        return [static_resource_uri(self.engine.js_path)] + self._script_files
